#!/bin/bash

help() {
  >&2 echo '
s6 compatibility script of:

invoke-rc.d, Debian/SysVinit (/etc/rc?.d) initscript subsystem.

Usage:
  invoke-rc.d [options] <basename> <action> [extra parameters]

  basename - Initscript ID, as per update-rc.d(8)
  action   - Initscript action. Known actions are:
                start, [force-]stop, [try-]restart,
                [force-]reload, status
  WARNING: not all initscripts implement all of the above actions.

  extra parameters are passed as is to the initscript, following 
  the action (first initscript parameter).

Options:
  --quiet
     Quiet mode, no error messages are generated.
  --force
     Try to run the initscript regardless of policy and subsystem
     non-fatal errors.
  --try-anyway
     Try to run init script even if a non-fatal error is found.
  --disclose-deny
     Return status code 101 instead of status code 0 if
     initscript action is denied by local policy rules or
     runlevel constrains.
  --query
     Returns one of status codes 100-106, does not run
     the initscript. Implies --disclose-deny and --no-fallback.
  --no-fallback
     Ignores any fallback action requests by the policy layer.
     Warning: this is usually a very *bad* idea for any actions
     other than "start".
  --skip-systemd-native
    Exits before doing anything if a systemd environment is detected
    and the requested service is a native systemd unit.
    This is useful for maintainer scripts that want to defer systemd
    actions to deb-systemd-invoke
  --help
     Outputs help message to stdout
'
}

# discard option parameters
# if a parameter --query is encountered: exit with status code 105
# if a parameter --help is encountered: display usage information
while [[ $1 =~ ^-- ]]; do
  case "$1" in
    --help)  help; exit 0;;
    --query) exit 105;;
  esac
  shift
done

# need at least 2 non-option arguments (service name and action)
if [ $# -lt 2 ]; then
  help
  exit 1
fi

# only manage service if it's known to s6
if svctl listall | grep -q "^${1}$"; then
  case "$2" in
    start)
      # Using "apply" instead of "start" restores all services to boot
      # configuration. Since dependencies are automatically stopped too when a
      # service is stopped, we want them to be automatically started again when
      # a package uses invoke-rc.d stop/start instead of restart. Services that
      # have been disabled (in the boot config) are automatically exempted.
      svctl apply || exit 1
      ;;
    stop|restart)
      svctl "$2" "$1" || exit 1
      ;;
    force-stop)
      svctl stop "$1" || exit 1
      ;;
    try-restart|reload|force-reload)
      svctl restart "$1" || exit 1
      ;;
    status)
      svctl isenabled "$1"
      svctl isup "$1"
      ;;
    *)
      help
      exit 1
      ;;
  esac
else
  echo "Service ${1} is unknown to s6."
fi

exit 0
